@@ -6,8 +6,8 @@
 
 #include "mozilla/dom/MessageManagerGlobal.h"
 #include "mozilla/IntentionalCrash.h"
-#include "mozilla/dom/DOMPrefs.h"
 #include "nsContentUtils.h"
+#include "nsJSUtils.h"
 
 #ifdef ANDROID
 #  include <android/log.h>
@@ -20,7 +20,7 @@ namespace mozilla {
 namespace dom {
 
 void MessageManagerGlobal::Dump(const nsAString& aStr) {
-  if (!DOMPrefs::DumpEnabled()) {
+  if (!nsJSUtils::DumpEnabled()) {
     return;
   }
 
@@ -37,14 +37,6 @@ void MessageManagerGlobal::Dump(const nsAString& aStr) {
   fflush(stdout);
 }
 
-void MessageManagerGlobal::PrivateNoteIntentionalCrash(ErrorResult& aError) {
-  if (XRE_IsContentProcess()) {
-    NoteIntentionalCrash("tab");
-    return;
-  }
-  aError.Throw(NS_ERROR_NOT_IMPLEMENTED);
-}
-
 void MessageManagerGlobal::Atob(const nsAString& aAsciiString,
                                 nsAString& aBase64Data, ErrorResult& aError) {
   aError = nsContentUtils::Atob(aAsciiString, aBase64Data);
